@@ -81,11 +81,11 @@ namespace xsimd
             static inline B compute(const B& x)
             {
                 return horner<B,
-                              0x3fb5555555555986ll,  //   8.33333333333482257126E-2
-                              0x3f6c71c71b98c5fdll,  //   3.47222221605458667310E-3
-                              0xbf65f72607d44fd7ll,  //  -2.68132617805781232825E-3
-                              0xbf2e166b27e61d7cll,  //  -2.29549961613378126380E-4
-                              0x3f49cc72592d7293ll   //   7.87311395793093628397E-4
+                              0x3fb5555555555986ull,  //   8.33333333333482257126E-2
+                              0x3f6c71c71b98c5fdull,  //   3.47222221605458667310E-3
+                              0xbf65f72607d44fd7ull,  //  -2.68132617805781232825E-3
+                              0xbf2e166b27e61d7cull,  //  -2.29549961613378126380E-4
+                              0x3f49cc72592d7293ull   //   7.87311395793093628397E-4
                               >(x);
             }
 
@@ -351,32 +351,32 @@ namespace xsimd
             static inline B gammaln1(const B& x)
             {
                 return horner<B,
-                              0xc12a0c675418055ell,  //  -8.53555664245765465627E5
-                              0xc13a45890219f20bll,  //  -1.72173700820839662146E6,
-                              0xc131bc82f994db51ll,  //  -1.16237097492762307383E6,
-                              0xc1143d73f89089e5ll,  //  -3.31612992738871184744E5,
-                              0xc0e2f234355bb93ell,  //  -3.88016315134637840924E4,
-                              0xc09589018ff36761ll  //  -1.37825152569120859100E3
+                              0xc12a0c675418055eull,  //  -8.53555664245765465627E5
+                              0xc13a45890219f20bull,  //  -1.72173700820839662146E6,
+                              0xc131bc82f994db51ull,  //  -1.16237097492762307383E6,
+                              0xc1143d73f89089e5ull,  //  -3.31612992738871184744E5,
+                              0xc0e2f234355bb93eull,  //  -3.88016315134637840924E4,
+                              0xc09589018ff36761ull  //  -1.37825152569120859100E3
                               >(x) /
                     horner<B,
-                           0xc13ece4b6a11e14all,  //  -2.01889141433532773231E6
-                           0xc1435255892ff34cll,  //  -2.53252307177582951285E6,
-                           0xc131628671950043ll,  //  -1.13933444367982507207E6,
-                           0xc10aeb84b9744c9bll,  //  -2.20528590553854454839E5,
-                           0xc0d0aa0d7b89d757ll,  //  -1.70642106651881159223E4,
-                           0xc075fd0d1cf312b2ll,  //  -3.51815701436523470549E2,
-                           0x3ff0000000000000ll  //   1.00000000000000000000E0
+                           0xc13ece4b6a11e14aull,  //  -2.01889141433532773231E6
+                           0xc1435255892ff34cull,  //  -2.53252307177582951285E6,
+                           0xc131628671950043ull,  //  -1.13933444367982507207E6,
+                           0xc10aeb84b9744c9bull,  //  -2.20528590553854454839E5,
+                           0xc0d0aa0d7b89d757ull,  //  -1.70642106651881159223E4,
+                           0xc075fd0d1cf312b2ull,  //  -3.51815701436523470549E2,
+                           0x3ff0000000000000ull  //   1.00000000000000000000E0
                            >(x);
             }
 
             static inline B gammalnA(const B& x)
             {
                 return horner<B,
-                              0x3fb555555555554bll,  //    8.33333333333331927722E-2
-                              0xbf66c16c16b0a5a1ll,  //   -2.77777777730099687205E-3,
-                              0x3f4a019f20dc5ebbll,  //    7.93650340457716943945E-4,
-                              0xbf437fbdb580e943ll,  //   -5.95061904284301438324E-4,
-                              0x3f4a985027336661ll  //    8.11614167470508450300E-4
+                              0x3fb555555555554bull,  //    8.33333333333331927722E-2
+                              0xbf66c16c16b0a5a1ull,  //   -2.77777777730099687205E-3,
+                              0x3f4a019f20dc5ebbull,  //    7.93650340457716943945E-4,
+                              0xbf437fbdb580e943ull,  //   -5.95061904284301438324E-4,
+                              0x3f4a985027336661ull  //    8.11614167470508450300E-4
                               >(x);
             }
         };
