@@ -863,9 +863,9 @@ namespace xsimd
                 return result;
             }
 
-            static batch_type haddp(const simd_batch<batch_type>* row)
+            static batch_type haddp(const batch_type* row)
             {
-                XSIMD_FALLBACK_MAPPING_LOOP(batch, hadd(row[i]()))
+                XSIMD_FALLBACK_MAPPING_LOOP(batch, hadd(row[i]))
             }
 
             static batch_type select(const batch_bool_type& cond, const batch_type& a, const batch_type& b)
