@@ -634,17 +634,17 @@ namespace xsimd
                 return _mm_cvtss_f32(tmp1);
             }
 
-            static batch_type haddp(const simd_batch<batch_type>* row)
+            static batch_type haddp(const batch_type* row)
             {
 #if XSIMD_X86_INSTR_SET >= XSIMD_X86_SSE3_VERSION
-                return _mm_hadd_ps(_mm_hadd_ps(row[0](), row[1]()),
-                    _mm_hadd_ps(row[2](), row[3]()));
+                return _mm_hadd_ps(_mm_hadd_ps(row[0], row[1]),
+                    _mm_hadd_ps(row[2], row[3]));
 #else
-                __m128 tmp0 = _mm_unpacklo_ps(row[0](), row[1]());
-                __m128 tmp1 = _mm_unpackhi_ps(row[0](), row[1]());
-                __m128 tmp2 = _mm_unpackhi_ps(row[2](), row[3]());
+                __m128 tmp0 = _mm_unpacklo_ps(row[0], row[1]);
+                __m128 tmp1 = _mm_unpackhi_ps(row[0], row[1]);
+                __m128 tmp2 = _mm_unpackhi_ps(row[2], row[3]);
                 tmp0 = _mm_add_ps(tmp0, tmp1);
-                tmp1 = _mm_unpacklo_ps(row[2](), row[3]());
+                tmp1 = _mm_unpacklo_ps(row[2], row[3]);
                 tmp1 = _mm_add_ps(tmp1, tmp2);
                 tmp2 = _mm_movehl_ps(tmp1, tmp0);
                 tmp0 = _mm_movelh_ps(tmp0, tmp1);
