@@ -240,12 +240,18 @@ namespace xt
         }
 
         template <class E1, class E2>
-        inline bool is_linear_assign(const E1& e1, const E2& e2)
+        inline auto is_linear_assign(const E1& e1, const E2& e2) -> std::enable_if_t<has_strides<E1>::value, bool>
         {
             return (E1::contiguous_layout && E2::contiguous_layout && linear_static_layout<E1, E2>()) ||
                    (e1.layout() != layout_type::dynamic && e2.has_linear_assign(e1.strides()));
         }
 
+        template <class E1, class E2>
+        inline auto is_linear_assign(const E1&, const E2&) -> std::enable_if_t<!has_strides<E1>::value, bool>
+        {
+            return false;
+        }
+
         template <class E, class = void>
         struct has_step_leading : std::false_type
         {
@@ -286,8 +292,8 @@ namespace xt
         // scope of these data members (since they are odr-used).
         static constexpr bool contiguous_layout() { return E1::contiguous_layout && E2::contiguous_layout; }
         static constexpr bool convertible_types() { return std::is_convertible<typename E2::value_type, typename E1::value_type>::value; }
-        static constexpr bool lhs_simd_size() { return xsimd::simd_traits<typename E1::value_type>::size > 1; }
-        static constexpr bool rhs_simd_size() { return xsimd::simd_traits<typename E2::value_type>::size > 1; }
+        static constexpr bool lhs_simd_size() { return xt_simd::simd_traits<typename E1::value_type>::size > 1; }
+        static constexpr bool rhs_simd_size() { return xt_simd::simd_traits<typename E2::value_type>::size > 1; }
         static constexpr bool simd_size() { return lhs_simd_size() && rhs_simd_size(); }
         static constexpr bool forbid_simd() { return !has_simd_interface<E2>::value; }
         static constexpr bool simd_assign() { return contiguous_layout() && convertible_types() && simd_size() && has_simd_interface<E2>::value; }
@@ -339,7 +345,7 @@ namespace xt
         const E2& de2 = e2.derived_cast();
 
         size_type dim = de2.dimension();
-        shape_type shape = xtl::make_sequence<shape_type>(dim, size_type(0));
+        shape_type shape = uninitialized_shape<shape_type>(dim);
         bool trivial_broadcast = de2.broadcast_shape(shape, true);
 
         if (dim > de1.dimension() || shape > de1.shape())
@@ -428,7 +434,7 @@ namespace xt
                 using index_type = xindex_type_t<typename E1::shape_type>;
                 using size_type = typename E1::size_type;
                 size_type size = e2.dimension();
-                index_type shape = xtl::make_sequence<index_type>(size, size_type(0));
+                index_type shape = uninitialized_shape<index_type>(size);
                 bool trivial_broadcast = e2.broadcast_shape(shape, true);
                 e1.resize(std::move(shape));
                 return trivial_broadcast;
@@ -511,16 +517,16 @@ namespace xt
     template <class E1, class E2>
     inline void linear_assigner<simd_assign>::run(E1& e1, const E2& e2)
     {
-        using lhs_align_mode = xsimd::container_alignment_t<E1>;
+        using lhs_align_mode = xt_simd::container_alignment_t<E1>;
         constexpr bool is_aligned = std::is_same<lhs_align_mode, aligned_mode>::value;
         using rhs_align_mode = std::conditional_t<is_aligned, inner_aligned_mode, unaligned_mode>;
         using value_type = std::common_type_t<typename E1::value_type, typename E2::value_type>;
-        using simd_type = xsimd::simd_type<value_type>;
+        using simd_type = xt_simd::simd_type<value_type>;
         using size_type = typename E1::size_type;
         size_type size = e1.size();
         constexpr size_type simd_size = simd_type::size;
 
-        size_type align_begin = is_aligned ? 0 : xsimd::get_alignment_offset(e1.data(), size, simd_size);
+        size_type align_begin = is_aligned ? 0 : xt_simd::get_alignment_offset(e1.data(), size, simd_size);
         size_type align_end = align_begin + ((size - align_begin) & ~(simd_size - 1));
 
         for (size_type i = 0; i < align_begin; ++i)
@@ -791,7 +797,7 @@ namespace xt
         // add this when we have std::array index!
         // std::fill(idx.begin(), idx.end(), 0);
         using value_type = std::common_type_t<typename E1::value_type, typename E2::value_type>;
-        using simd_type = xsimd::simd_type<value_type>;
+        using simd_type = xt_simd::simd_type<value_type>;
 
         std::size_t simd_size = inner_loop_size / simd_type::size;
         std::size_t simd_rest = inner_loop_size % simd_type::size;
