@@ -14,6 +14,7 @@
 #define XTENSOR_BUILDER_HPP
 
 #include <array>
+#include <chrono>
 #include <cmath>
 #include <cstddef>
 #include <functional>
@@ -25,6 +26,7 @@
 
 #include <xtl/xclosure.hpp>
 #include <xtl/xsequence.hpp>
+#include <xtl/xtype_traits.hpp>
 
 #include "xbroadcast.hpp"
 #include "xfunction.hpp"
@@ -121,6 +123,12 @@ namespace xt
         using shape_type = typename xtensor<T, N>::shape_type;
         return xtensor<T, N, L>(xtl::forward_sequence<shape_type, decltype(shape)>(shape));
     }
+#else
+    template <class T, layout_type L = XTENSOR_DEFAULT_LAYOUT, class I>
+    inline xarray<T, L> empty(const std::initializer_list<I>& init)
+    {
+        return xarray<T, L>::from_shape(init);
+    }
 #endif
 
     template <class T, layout_type L = XTENSOR_DEFAULT_LAYOUT, std::size_t... N>
@@ -138,7 +146,7 @@ namespace xt
     template <class E>
     inline auto empty_like(const xexpression<E>& e)
     {
-        using xtype = detail::temporary_type_t<typename E::value_type, typename E::shape_type, E::static_layout>;
+        using xtype = temporary_type_t<typename E::value_type, typename E::shape_type, E::static_layout>;
         auto res = xtype::from_shape(e.derived_cast().shape());
         return res;
     }
@@ -153,7 +161,7 @@ namespace xt
     template <class E>
     inline auto full_like(const xexpression<E>& e, typename E::value_type fill_value)
     {
-        using xtype = detail::temporary_type_t<typename E::value_type, typename E::shape_type, E::static_layout>;
+        using xtype = temporary_type_t<typename E::value_type, typename E::shape_type, E::static_layout>;
         auto res = xtype::from_shape(e.derived_cast().shape());
         res.fill(fill_value);
         return res;
@@ -191,12 +199,54 @@ namespace xt
 
     namespace detail
     {
-        template <class T, class S = T>
+        template <class T, class S>
+        struct get_mult_type_impl
+        {
+            using type = T;
+        };
+
+        template <class T, class R, class P>
+        struct get_mult_type_impl<T, std::chrono::duration<R, P>>
+        {
+            using type = R;
+        };
+
+        template <class T, class S>
+        using get_mult_type = typename get_mult_type_impl<T, S>::type;
+
+        // These methods should be private methods of arange_generator, however thi leads
+        // to ICE on VS2015
+        template <class R, class E, class U, class X, XTL_REQUIRES(std::is_integral<X>)>
+        inline void arange_assign_to(xexpression<E>& e, U start, X step) noexcept
+        {
+            auto& de = e.derived_cast();
+            U value = start;
+
+            for (auto&& el : de.storage())
+            {
+                el = static_cast<R>(value);
+                value += step;
+            }
+        }
+
+        template <class R, class E, class U, class X, XTL_REQUIRES(xtl::negation<std::is_integral<X>>)>
+        inline void arange_assign_to(xexpression<E>& e, U start, X step) noexcept
+        {
+            auto& buf = e.derived_cast().storage();
+            using size_type = decltype(buf.size());
+            using mult_type = get_mult_type<U, X>;
+            for(size_type i = 0; i < buf.size(); ++i)
+            {
+                buf[i] = static_cast<R>(start + step * mult_type(i));
+            }
+        }
+
+        template <class T, class R = T, class S = T>
         class arange_generator
         {
         public:
 
-            using value_type = T;
+            using value_type = R;
             using step_type = S;
 
             arange_generator(T start, T stop, S step)
@@ -205,45 +255,41 @@ namespace xt
             }
 
             template <class... Args>
-            inline T operator()(Args... args) const
+            inline R operator()(Args... args) const
             {
                 return access_impl(args...);
             }
 
             template <class It>
-            inline T element(It first, It) const
+            inline R element(It first, It) const
             {
-                return m_start + m_step * T(*first);
+                // Avoids warning when T = char (because char + char => int!)
+                using mult_type = get_mult_type<T, S>;
+                return static_cast<R>(m_start + m_step * mult_type(*first));
             }
 
             template <class E>
             inline void assign_to(xexpression<E>& e) const noexcept
             {
-                auto& de = e.derived_cast();
-                value_type value = m_start;
-
-                for (auto&& el : de.storage())
-                {
-                    el = value;
-                    value += m_step;
-                }
+                arange_assign_to<R>(e, m_start, m_step);
             }
 
         private:
 
-            value_type m_start;
-            value_type m_stop;
+            T m_start;
+            T m_stop;
             step_type m_step;
 
             template <class T1, class... Args>
-            inline T access_impl(T1 t, Args...) const
+            inline R access_impl(T1 t, Args...) const
             {
-                return m_start + m_step * T(t);
+                using mult_type = get_mult_type<T, S>;
+                return static_cast<R>(m_start + m_step * mult_type(t));
             }
 
-            inline T access_impl() const
+            inline R access_impl() const
             {
-                return m_start;
+                return static_cast<R>(m_start);
             }
         };
 
@@ -254,14 +300,20 @@ namespace xt
         inline auto arange_impl(T start, T stop, S step = 1) noexcept
         {
             std::size_t shape = static_cast<std::size_t>(std::ceil((stop - start) / step));
-            return detail::make_xgenerator(detail::arange_generator<T, S>(start, stop, step), {shape});
+            return detail::make_xgenerator(detail::arange_generator<T, T, S>(start, stop, step), {shape});
         }
 
         template <class T, class S = T, XTL_REQUIRES(both_integer<T, S>)>
         inline auto arange_impl(T start, T stop, S step = 1) noexcept
         {
-            std::size_t shape = static_cast<std::size_t>((stop - start + step - S(1)) / step);
-            return detail::make_xgenerator(detail::arange_generator<T, S>(start, stop, step), {shape});
+            bool empty_cond = (stop - start) / step <= 0;
+            std::size_t shape = 0;
+            if(!empty_cond)
+            {
+                shape = stop > start ? static_cast<std::size_t>((stop - start + step - S(1)) / step)
+                                     : static_cast<std::size_t>((start - stop - step - S(1)) / -step);
+            }
+            return detail::make_xgenerator(detail::arange_generator<T, T, S>(start, stop, step), {shape});
         }
 
         template <class F>
@@ -401,8 +453,8 @@ namespace xt
     inline auto linspace(T start, T stop, std::size_t num_samples = 50, bool endpoint = true) noexcept
     {
         using fp_type = std::common_type_t<T, double>;
-        fp_type step = fp_type(stop - start) / fp_type(num_samples - (endpoint ? 1 : 0));
-        return cast<T>(detail::make_xgenerator(detail::arange_generator<fp_type>(fp_type(start), fp_type(stop), step), {num_samples}));
+        fp_type step = fp_type(stop - start) / std::fmax(fp_type(1), fp_type(num_samples - (endpoint ? 1 : 0)));
+        return detail::make_xgenerator(detail::arange_generator<fp_type, T>(fp_type(start), fp_type(stop), step), {num_samples});
     }
 
     /**
@@ -418,7 +470,7 @@ namespace xt
     template <class T>
     inline auto logspace(T start, T stop, std::size_t num_samples, T base = 10, bool endpoint = true) noexcept
     {
-        return cast<T>(pow(std::move(base), linspace(start, stop, num_samples, endpoint)));
+        return pow(std::move(base), linspace(start, stop, num_samples, endpoint));
     }
 
     namespace detail
@@ -429,7 +481,7 @@ namespace xt
         public:
 
             using size_type = std::size_t;
-            using value_type = promote_type_t<typename std::decay_t<CT>::value_type...>;
+            using value_type = xtl::promote_type_t<typename std::decay_t<CT>::value_type...>;
 
             inline concatenate_impl(std::tuple<CT...>&& t, size_type axis)
                 : m_t(t), m_axis(axis)
@@ -488,7 +540,7 @@ namespace xt
         public:
 
             using size_type = std::size_t;
-            using value_type = promote_type_t<typename std::decay_t<CT>::value_type...>;
+            using value_type = xtl::promote_type_t<typename std::decay_t<CT>::value_type...>;
 
             inline stack_impl(std::tuple<CT...>&& t, size_type axis)
                 : m_t(t), m_axis(axis)
@@ -591,10 +643,26 @@ namespace xt
         using shape_type = promote_shape_t<typename std::decay_t<CT>::shape_type...>;
         using source_shape_type = decltype(std::get<0>(t).shape());
         shape_type new_shape = xtl::forward_sequence<shape_type, source_shape_type>(std::get<0>(t).shape());
+        
+        auto check_shape = [&axis, &new_shape](auto& arr) {
+            std::size_t s = new_shape.size();
+            bool res = s == arr.dimension();
+            for(std::size_t i = 0; i < s; ++i)
+            {
+                res = res && (i == axis || new_shape[i] == arr.shape(i));
+            }
+            if(!res)
+            {
+                throw_concatenate_error(new_shape, arr.shape());
+            }
+        };
+        for_each(check_shape, t);
+
         auto shape_at_axis = [&axis](std::size_t prev, auto& arr) -> std::size_t {
             return prev + arr.shape()[axis];
         };
         new_shape[axis] += accumulate(shape_at_axis, std::size_t(0), t) - new_shape[axis];
+        
         return detail::make_xgenerator(detail::concatenate_impl<CT...>(std::forward<std::tuple<CT...>>(t), axis), new_shape);
     }
 
