@@ -111,7 +111,7 @@ namespace xt
         using difference_type = typename subiterator_traits::difference_type;
         using size_type = typename storage_type::size_type;
         using shape_type = typename storage_type::shape_type;
-        using simd_type = xsimd::simd_type<value_type>;
+        using simd_type = xt_simd::simd_type<value_type>;
 
         xstepper() = default;
         xstepper(storage_type* c, subiterator_type it, size_type offset) noexcept;
@@ -129,7 +129,7 @@ namespace xt
         template <class R>
         R step_simd();
 
-        value_type step_leading();
+        void step_leading();
 
         template <class R>
         void store_simd(const R& vec);
@@ -523,7 +523,7 @@ namespace xt
     {
         R reg;
         reg.load_unaligned(&(*m_it));
-        m_it += xsimd::revert_simd_traits<R>::size;
+        m_it += xt_simd::revert_simd_traits<R>::size;
         return reg;
     }
 
@@ -532,14 +532,13 @@ namespace xt
     inline void xstepper<C>::store_simd(const R& vec)
     {
         vec.store_unaligned(&(*m_it));
-        m_it += xsimd::revert_simd_traits<R>::size;;
+        m_it += xt_simd::revert_simd_traits<R>::size;;
     }
 
     template <class C>
-    auto xstepper<C>::step_leading() -> value_type
+    void xstepper<C>::step_leading()
     {
         ++m_it;
-        return *m_it;
     }
 
     template <>
@@ -614,7 +613,7 @@ namespace xt
                 }
             }
         }
-        if (i == 0)
+        if (i == 0 && n != 0)
         {
             std::copy(shape.cbegin(), shape.cend(), index.begin());
             stepper.to_end(layout_type::row_major);
@@ -692,7 +691,7 @@ namespace xt
                 }
             }
         }
-        if (i == 0)
+        if (i == 0 && n != 0)
         {
             stepper.to_begin();
         }
@@ -773,7 +772,7 @@ namespace xt
             }
             ++i;
         }
-        if (i == size)
+        if (i == size && n != 0)
         {
             std::copy(shape.cbegin(), shape.cend(), index.begin());
             stepper.to_end(layout_type::column_major);
@@ -854,7 +853,7 @@ namespace xt
             }
             ++i;
         }
-        if (i == size)
+        if (i == size && n != 0)
         {
             stepper.to_begin();
         }
