@@ -21,6 +21,73 @@
 
 namespace xt
 {
+    namespace detail
+    {
+        template <class CT, layout_type L>
+        class flat_expression_adaptor
+        {
+        public:
+
+            using xexpression_type = std::decay_t<CT>;
+            using shape_type = typename xexpression_type::shape_type;
+            using inner_strides_type = get_strides_t<shape_type>;
+            using index_type = inner_strides_type;
+            using size_type = typename xexpression_type::size_type;
+            using value_type = typename xexpression_type::value_type;
+            using const_reference = typename xexpression_type::const_reference;
+            using reference = std::conditional_t<std::is_const<std::remove_reference_t<CT>>::value,
+                                                 typename xexpression_type::const_reference,
+                                                 typename xexpression_type::reference>;
+
+            using iterator = decltype(std::declval<std::remove_reference_t<CT>>().template begin<L>());
+            using const_iterator = decltype(std::declval<std::decay_t<CT>>().template cbegin<L>());
+
+            explicit flat_expression_adaptor(CT* e);
+
+            template <class FST>
+            flat_expression_adaptor(CT* e, FST&& strides);
+
+            void update_pointer(CT* ptr) const
+            {
+                m_e = ptr;
+            }
+
+            size_type size() const;
+            reference operator[](size_type idx);
+            const_reference operator[](size_type idx) const;
+
+            iterator begin();
+            iterator end();
+            const_iterator begin() const;
+            const_iterator end() const;
+            const_iterator cbegin() const;
+            const_iterator cend() const;
+
+        private:
+
+            mutable CT* m_e;
+            inner_strides_type m_strides;
+            mutable index_type m_index;
+            size_type m_size;
+        };
+
+        template <class T>
+        struct is_flat_expression_adaptor : std::false_type
+        {
+        };
+
+        template <class CT, layout_type L>
+        struct is_flat_expression_adaptor<flat_expression_adaptor<CT, L>> : std::true_type
+        {
+        };
+
+        template <class E, class ST>
+        struct provides_data_interface
+            : xtl::conjunction<has_data_interface<std::decay_t<E>>, xtl::negation<is_flat_expression_adaptor<ST>>>
+        {
+        };
+    }
+
     template <class D>
     class xstrided_view_base : public xaccessible<D>
     {
@@ -42,6 +109,7 @@ namespace xt
         using size_type = typename inner_types::size_type;
         using difference_type = typename xexpression_type::difference_type;
 
+        using storage_getter = typename inner_types::storage_getter;
         using inner_storage_type = typename inner_types::inner_storage_type;
         using storage_type = std::remove_reference_t<inner_storage_type>;
 
@@ -58,12 +126,8 @@ namespace xt
         static constexpr layout_type static_layout = inner_types::layout;
         static constexpr bool contiguous_layout = static_layout != layout_type::dynamic && xexpression_type::contiguous_layout;
 
-        template <class CTA>
-        xstrided_view_base(CTA&& e, undecay_shape&& shape, strides_type&& strides, size_type offset, layout_type layout) noexcept;
-
-        template <class CTA, class FLS>
-        xstrided_view_base(CTA&& e, undecay_shape&& shape, strides_type&& strides, size_type offset,
-                           layout_type layout, FLS&& flatten_strides, layout_type flatten_layout) noexcept;
+        template <class CTA, class SA>
+        xstrided_view_base(CTA&& e, SA&& shape, strides_type&& strides, size_type offset, layout_type layout) noexcept;
 
         xstrided_view_base(xstrided_view_base&& rhs);
 
@@ -73,6 +137,7 @@ namespace xt
         const inner_strides_type& strides() const noexcept;
         const inner_backstrides_type& backstrides() const noexcept;
         layout_type layout() const noexcept;
+        using base_type::shape;
 
         reference operator()();
         const_reference operator()() const;
@@ -98,11 +163,11 @@ namespace xt
         storage_type& storage() noexcept;
         const storage_type& storage() const noexcept;
 
-        template <class E = xexpression_type>
-        std::enable_if_t<has_data_interface<std::decay_t<E>>::value, value_type*>
+        template <class E = xexpression_type, class ST = storage_type>
+        std::enable_if_t<detail::provides_data_interface<E, ST>::value, pointer>
         data() noexcept;
-        template <class E = xexpression_type>
-        std::enable_if_t<has_data_interface<std::decay_t<E>>::value, const value_type*>
+        template <class E = xexpression_type, class ST = storage_type>
+        std::enable_if_t<detail::provides_data_interface<E, ST>::value, const_pointer>
         data() const noexcept;
         size_type data_offset() const noexcept;
 
@@ -146,121 +211,74 @@ namespace xt
     namespace detail
     {
         template <class CT>
-        class flat_expression_adaptor
+        struct inner_storage_getter
         {
-        public:
+            using type = decltype(std::declval<CT>().storage());
+            using reference = std::add_lvalue_reference_t<CT>;
 
-            using xexpression_type = std::decay_t<CT>;
-            using shape_type = typename xexpression_type::shape_type;
-            using inner_strides_type = get_strides_t<shape_type>;
-            using index_type = inner_strides_type;
-            using size_type = typename xexpression_type::size_type;
-            using value_type = typename xexpression_type::value_type;
-            using reference = typename xexpression_type::reference;
-            using const_reference = typename xexpression_type::const_reference;
+            template <class E>
+            using rebind_t = inner_storage_getter<E>;
 
-            using iterator = decltype(std::declval<std::decay_t<CT>>().template begin<XTENSOR_DEFAULT_LAYOUT>());
-            using const_iterator = decltype(std::declval<std::decay_t<CT>>().template cbegin<XTENSOR_DEFAULT_LAYOUT>());
-
-            explicit flat_expression_adaptor(CT* e);
-
-            template <class FST>
-            flat_expression_adaptor(CT* e, FST&& strides, layout_type layout);
-
-            void update_pointer(CT* ptr) const
+            static decltype(auto) get_flat_storage(reference e)
             {
-                m_e = ptr;
+                return e.storage();
             }
 
-            size_type size() const;
-            reference operator[](size_type idx);
-            const_reference operator[](size_type idx) const;
+            static auto get_offset(reference e)
+            {
+                return e.data_offset();
+            }
 
-            iterator begin();
-            iterator end();
-            const_iterator begin() const;
-            const_iterator end() const;
-            const_iterator cbegin() const;
-            const_iterator cend() const;
-
-        private:
-
-            mutable CT* m_e;
-            inner_strides_type m_strides;
-            mutable index_type m_index;
-            size_type m_size;
-            layout_type m_layout;
+            static decltype(auto) get_strides(reference e)
+            {
+                return e.strides();
+            }
         };
 
-        template <class CT, class T = void>
-        struct flat_storage_type;
-
-        template <class CT>
-        struct flat_storage_type<CT, typename std::enable_if_t<has_data_interface<std::decay_t<CT>>::value>>
+        template <class CT, layout_type L>
+        struct flat_adaptor_getter
         {
-            // Note: we could also use the storage_type typedef.
-            // using type = std::conditional_t<
-            //    std::is_const<std::remove_reference_t<CT>>::value,
-            //    const typename std::decay_t<CT>::storage_type&,
-            //    typename std::decay_t<CT>::storage_type&>;
-            using type = decltype(std::declval<CT>().storage());
+            using type = flat_expression_adaptor<std::remove_reference_t<CT>, L>;
+            using reference = std::add_lvalue_reference_t<CT>;
+
+            template <class E>
+            using rebind_t = flat_adaptor_getter<E, L>;
+
+            static type get_flat_storage(reference e)
+            {
+                // moved to addressof because ampersand on xview returns a closure pointer
+                return type(std::addressof(e));
+            }
+
+            static auto get_offset(reference)
+            {
+                return typename std::decay_t<CT>::size_type(0);
+            }
+
+            static auto get_strides(reference e)
+            {
+                dynamic_shape<std::ptrdiff_t> strides;
+                strides.resize(e.shape().size());
+                compute_strides(e.shape(), L, strides);
+                return strides;
+            }
         };
 
-        template <class CT>
-        struct flat_storage_type<CT, typename std::enable_if_t<!has_data_interface<std::decay_t<CT>>::value>>
-        {
-            using type = flat_expression_adaptor<std::remove_reference_t<CT>>;
-        };
+        template <class CT, layout_type L>
+        using flat_storage_getter = std::conditional_t<has_data_interface<std::decay_t<CT>>::value,
+                                                       inner_storage_getter<CT>,
+                                                       flat_adaptor_getter<CT, L>>;
 
-        template <class CT>
-        using flat_storage_type_t = typename flat_storage_type<CT>::type;
-
-        // with data_interface
-        template <class E, std::enable_if_t<has_data_interface<std::decay_t<E>>::value>* = nullptr>
-        inline decltype(auto) get_flat_storage(E& e)
+        template <layout_type L, class E>
+        inline auto get_offset(E& e)
         {
-            return e.storage();
+            return flat_storage_getter<E, L>::get_offset(e);
         }
 
-        template <class E, std::enable_if_t<has_data_interface<std::decay_t<E>>::value>* = nullptr>
-        inline auto get_offset(E&& e)
+        template <layout_type L, class E>
+        inline decltype(auto) get_strides(E& e)
         {
-            return e.data_offset();
-        }
-
-        template <class E, std::enable_if_t<has_data_interface<std::decay_t<E>>::value>* = nullptr>
-        inline decltype(auto) get_strides(E&& e)
-        {
-            return e.strides();
-        }
-
-        // without data_interface
-        template <class E, std::enable_if_t<!has_data_interface<std::decay_t<E>>::value>* = nullptr>
-        inline auto get_flat_storage(E& e) -> flat_expression_adaptor<std::remove_reference_t<E>>
-        {
-            // moved to addressof because ampersand on xview returns a closure pointer
-            return flat_expression_adaptor<std::remove_reference_t<E>>(std::addressof(e));
-        }
-
-        template <class E, class S>
-        inline auto get_flat_storage(E& e, S&& s, layout_type l) -> flat_expression_adaptor<std::remove_reference_t<E>>
-        {
-            return flat_expression_adaptor<std::remove_reference_t<E>>(std::addressof(e), std::forward<S>(s), l);
-        }
-
-        template <class E, std::enable_if_t<!has_data_interface<std::decay_t<E>>::value>* = nullptr>
-        inline auto get_offset(E&& /*e*/)
-        {
-            return typename std::decay_t<E>::size_type(0);
-        }
-
-        template <class E, std::enable_if_t<!has_data_interface<std::decay_t<E>>::value>* = nullptr>
-        inline auto get_strides(E&& e)
-        {
-            dynamic_shape<std::ptrdiff_t> strides;
-            strides.resize(e.shape().size());
-            compute_strides(e.shape(), XTENSOR_DEFAULT_LAYOUT, strides);
-            return strides;
+            return flat_storage_getter<E, L>::get_strides(e);
         }
     }
 
@@ -282,27 +300,12 @@ namespace xt
      * @param layout the layout of the view
      */
     template <class D>
-    template <class CTA>
-    inline xstrided_view_base<D>::xstrided_view_base(CTA&& e, undecay_shape&& shape, strides_type&& strides, size_type offset, layout_type layout) noexcept
+    template <class CTA, class SA>
+    inline xstrided_view_base<D>::xstrided_view_base(CTA&& e, SA&& shape, strides_type&& strides, size_type offset, layout_type layout) noexcept
         : m_e(std::forward<CTA>(e)),
-          m_storage(detail::get_flat_storage<undecay_expression>(m_e)),
-          m_shape(std::move(shape)),
-          m_strides(std::move(strides)),
-          m_offset(offset),
-          m_layout(layout)
-    {
-        m_backstrides = xtl::make_sequence<backstrides_type>(m_shape.size(), 0);
-        adapt_strides(m_shape, m_strides, m_backstrides);
-    }
-
-    template <class D>
-    template <class CTA, class FLS>
-    inline xstrided_view_base<D>::xstrided_view_base(CTA&& e, undecay_shape&& shape, strides_type&& strides,
-                                                     size_type offset, layout_type layout,
-                                                     FLS&& flatten_strides, layout_type flatten_layout) noexcept
-        : m_e(std::forward<CTA>(e)),
-          m_storage(detail::get_flat_storage<undecay_expression>(m_e, std::forward<FLS>(flatten_strides), flatten_layout)),
-          m_shape(std::move(shape)),
+          //m_storage(detail::get_flat_storage<undecay_expression>(m_e)),
+          m_storage(storage_getter::get_flat_storage(m_e)),
+          m_shape(std::forward<SA>(shape)),
           m_strides(std::move(strides)),
           m_offset(offset),
           m_layout(layout)
@@ -319,10 +322,10 @@ namespace xt
             return expr.storage();
         }
 
-        template <class T, class E>
-        auto copy_move_storage(T& expr, const detail::flat_expression_adaptor<E>& storage)
+        template <class T, class E, layout_type L>
+        auto copy_move_storage(T& expr, const detail::flat_expression_adaptor<E, L>& storage)
         {
-            detail::flat_expression_adaptor<E> new_storage = storage; // copy storage
+            detail::flat_expression_adaptor<E, L> new_storage = storage; // copy storage
             new_storage.update_pointer(std::addressof(expr));
             return new_storage;
         }
@@ -551,9 +554,9 @@ namespace xt
      * The first element of the view is at data() + data_offset().
      */
     template <class D>
-    template <class E>
+    template <class E, class ST>
     inline auto xstrided_view_base<D>::data() noexcept ->
-        std::enable_if_t<has_data_interface<std::decay_t<E>>::value, value_type*>
+        std::enable_if_t<detail::provides_data_interface<E, ST>::value, pointer>
     {
         return m_e.data();
     }
@@ -563,9 +566,9 @@ namespace xt
      * The first element of the view is at data() + data_offset().
      */
     template <class D>
-    template <class E>
+    template <class E, class ST>
     inline auto xstrided_view_base<D>::data() const noexcept ->
-        std::enable_if_t<has_data_interface<std::decay_t<E>>::value, const value_type*>
+        std::enable_if_t<detail::provides_data_interface<E, ST>::value, const_pointer>
     {
         return m_e.data();
     }
@@ -656,82 +659,82 @@ namespace xt
 
     namespace detail
     {
-        template <class CT>
-        inline flat_expression_adaptor<CT>::flat_expression_adaptor(CT* e)
+        template <class CT, layout_type L>
+        inline flat_expression_adaptor<CT, L>::flat_expression_adaptor(CT* e)
             : m_e(e)
         {
             resize_container(m_index, m_e->dimension());
             resize_container(m_strides, m_e->dimension());
             m_size = compute_size(m_e->shape());
-            m_layout = XTENSOR_DEFAULT_LAYOUT;
-            compute_strides(m_e->shape(), m_layout, m_strides);
+            compute_strides(m_e->shape(), L, m_strides);
         }
 
-        template <class CT>
+        template <class CT, layout_type L>
         template <class FST>
-        inline flat_expression_adaptor<CT>::flat_expression_adaptor(CT* e, FST&& strides, layout_type layout)
-            : m_e(e), m_strides(xtl::forward_sequence<inner_strides_type, FST>(strides)), m_layout(layout)
+        inline flat_expression_adaptor<CT, L>::flat_expression_adaptor(CT* e, FST&& strides)
+            : m_e(e), m_strides(xtl::forward_sequence<inner_strides_type, FST>(strides))
         {
             resize_container(m_index, m_e->dimension());
             m_size = m_e->size();
         }
 
-        template <class CT>
-        inline auto flat_expression_adaptor<CT>::size() const -> size_type
+        template <class CT, layout_type L>
+        inline auto flat_expression_adaptor<CT, L>::size() const -> size_type
         {
             return m_size;
         }
 
-        template <class CT>
-        inline auto flat_expression_adaptor<CT>::operator[](size_type idx) -> reference
+
+        template <class CT, layout_type L>
+        inline auto flat_expression_adaptor<CT, L>::operator[](size_type idx) -> reference
         {
             auto i = static_cast<typename index_type::value_type>(idx);
-            m_index = detail::unravel_noexcept(i, m_strides, m_layout);
+            m_index = detail::unravel_noexcept(i, m_strides, L);
             return m_e->element(m_index.cbegin(), m_index.cend());
         }
 
-        template <class CT>
-        inline auto flat_expression_adaptor<CT>::operator[](size_type idx) const -> const_reference
+        template <class CT, layout_type L>
+        inline auto flat_expression_adaptor<CT, L>::operator[](size_type idx) const -> const_reference
         {
             auto i = static_cast<typename index_type::value_type>(idx);
-            m_index = detail::unravel_noexcept(i, m_strides, m_layout);
+            m_index = detail::unravel_noexcept(i, m_strides, L);
             return m_e->element(m_index.cbegin(), m_index.cend());
         }
 
-        template <class CT>
-        inline auto flat_expression_adaptor<CT>::begin() -> iterator
+        template <class CT, layout_type L>
+        inline auto flat_expression_adaptor<CT, L>::begin() -> iterator
         {
-            return m_e->template begin<XTENSOR_DEFAULT_LAYOUT>();
+            return m_e->template begin<L>();
         }
 
-        template <class CT>
-        inline auto flat_expression_adaptor<CT>::end() -> iterator
+        template <class CT, layout_type L>
+        inline auto flat_expression_adaptor<CT, L>::end() -> iterator
         {
-            return m_e->template end<XTENSOR_DEFAULT_LAYOUT>();
+            return m_e->template end<L>();
         }
 
-        template <class CT>
-        inline auto flat_expression_adaptor<CT>::begin() const -> const_iterator
+        template <class CT, layout_type L>
+        inline auto flat_expression_adaptor<CT, L>::begin() const -> const_iterator
         {
-            return m_e->template cbegin<XTENSOR_DEFAULT_LAYOUT>();
+            return m_e->template cbegin<L>();
         }
 
-        template <class CT>
-        inline auto flat_expression_adaptor<CT>::end() const -> const_iterator
+        template <class CT, layout_type L>
+        inline auto flat_expression_adaptor<CT, L>::end() const -> const_iterator
         {
-            return m_e->template cend<XTENSOR_DEFAULT_LAYOUT>();
+            return m_e->template cend<L>();
         }
 
-        template <class CT>
-        inline auto flat_expression_adaptor<CT>::cbegin() const -> const_iterator
+        template <class CT, layout_type L>
+        inline auto flat_expression_adaptor<CT, L>::cbegin() const -> const_iterator
         {
-            return m_e->template cbegin<XTENSOR_DEFAULT_LAYOUT>();
+            return m_e->template cbegin<L>();
         }
 
-        template <class CT>
-        inline auto flat_expression_adaptor<CT>::cend() const ->const_iterator
+        template <class CT, layout_type L>
+        inline auto flat_expression_adaptor<CT, L>::cend() const ->const_iterator
         {
-            return m_e->template cend<XTENSOR_DEFAULT_LAYOUT>();
+            return m_e->template cend<L>();
         }
     }
 
