@@ -21,6 +21,7 @@
 #endif
 
 #include "xhash.hpp"
+#include "xtl_config.hpp"
 
 namespace xtl
 {
@@ -111,7 +112,7 @@ namespace xtl
         {
             fixed_string_external_storage_impl() = default;
 
-            fixed_string_external_storage_impl(T ptr, std::ptrdiff_t size)
+            fixed_string_external_storage_impl(T ptr, std::ptrdiff_t/*size*/)
             {
                 m_buffer = ptr;
             }
@@ -675,7 +676,12 @@ namespace xtl
                 {
                     std::ostringstream oss;
                     oss << "Invalid size (" << size << ") for xbasic_fixed_string - maximal size: " << N;
+#if defined(XTL_NO_EXCEPTIONS)
+                    std::fprintf(stderr, "%s\n", oss.str().c_str());
+                    std::terminate();
+#else
                     throw std::length_error(oss.str());
+#endif
                 }
                 return size;
             }
@@ -1933,7 +1939,12 @@ namespace xtl
     {
         if (pos >= size)
         {
+#if defined(XTL_NO_EXCEPTIONS)
+            std::fprintf(stderr, "%s\n", what);
+            std::terminate();
+#else
             throw std::out_of_range(what);
+#endif
         }
     }
 
